/* GCC */

#include <iostream>
#include <fstream>
#include <vector>

const int width = 4;

template <int n>
struct Matrix
{
    enum {size = n};
    int matrix[n][n];
    int* operator[](int i)
    {
        return matrix[i];
    }
    const int* operator[](int i) const
    {
        return matrix[i];
    }
};

template <int n>
Matrix<n> matrix(int diagonal = 0)
{
    Matrix<n> answer;
    for(int i = 0;i < answer.size;i++)
    {
        for(int j = 0;j < answer.size;j++)
        {
            if(i == j)
                answer[i][j] = diagonal;
            else
                answer[i][j] = 0;
        }
    }
    return answer;
}

template <int n>
void productByMod(Matrix<n>& m1,const Matrix<n>& m2,int mod)
{
    Matrix<n> answer = matrix<n>();
    for(int i = 0;i < n;i++)
        for(int j = 0;j < n;j++)
            for(int k = 0;k < n;k++)
                answer[i][j] = (answer[i][j] + m1[i][k] * 1ll * m2[k][j]) % mod;
    m1 = answer;
}

template <int n>
Matrix<n> powerByMod(Matrix<n> m,int k,int mod)
{
    Matrix<n> answer = matrix<n>(1);
    while(k)
    {
        if(k & 1)
            productByMod(answer,m,mod);
        productByMod(m,m,mod);
        k >>= 1;
    }
    return answer;
}

struct Profile
{
    unsigned int mask[2];
    operator unsigned int*()
    {
        return mask;
    }
};

Profile profile(unsigned int m1,unsigned int m2)
{
    Profile answer = {m1,m2};
    return answer;
}

bool isValid(Profile p)
{
    for(int i = 0;i < width;i++)
        for(int j = 0;j < width;j++)
            if(std::abs(i-j) == 2 &&
               (p[0] & (1 << i)) &&
               (p[1] & (1 << j)))
                return false;
    return true;
}

bool areValid(Profile previous,Profile current)
{
    if(previous[1] != current[0])
        return false;
    for(int i = 0;i < width;i++)
        for(int j = 0;j < width;j++)
            if(std::abs(i-j) == 1 &&
               (previous[0] & (1 << i)) &&
               (current[1] & (1 << j)))
                return false;
    return true;
}

int main()
{
    std::vector<Profile> profile;
    for(unsigned int p1 = 0;p1 < (1 << width);p1++)
        for(unsigned int p2 = 0;p2 < (1 << width);p2++)
            if(isValid(::profile(p1,p2)))
                profile.push_back(::profile(p1,p2));
    if(profile.size() != 81)
        throw;
    enum {profileSize = 81};
    Matrix<profileSize> dynamic = matrix<profileSize>();
    for(int i = 0;i < profile.size();i++)
        for(int j = 0;j < profile.size();j++)
            if(areValid(profile[i],profile[j]))
                dynamic[j][i] = 1;
    std::ifstream in("chess.in");
    int n,p;
    in >> n >> p;
    in.close();
    Matrix<profileSize> last = powerByMod(dynamic,n-2,p);
    int answer = 0;
    for(int i = 0;i < last.size;i++)
        for(int j = 0;j < last.size;j++)
            answer = (answer + last[i][j]) % p;
    std::ofstream out("chess.out");
    out << answer << std::endl;
    out.close();
}
