/* GCC */
#include <cstdio>
#include <algorithm>

const int Infinity = 1e9;

int N , M , T[256][256] , D[256][256][4] , Heap[256 * 256 * 4] , Dist[256 * 256 * 4] , HeapSize;
bool Used[256][256][4];

struct state
{
    int X , Y , Corner;
    state () {};
    state (int _X , int _Y , int _Corner) : X(_X) , Y(_Y) , Corner(_Corner) {};
};

int StateToHash(state S)
{
  return S.X * M * 4 + S.Y * 4 + S.Corner;
};

state HashToState(int H)
{
  return state(H / 4 / M , H / 4 % M , H % 4);
};

void Insert(state S , int Dt)
{
  Heap[ ++HeapSize ] = StateToHash(S);
  Dist[HeapSize] = Dt;
  for (int v = HeapSize ; v / 2 ; v /= 2 )
    if ( Dist[v] < Dist[v / 2] )
    {
        Heap[v]^=Heap[v / 2]^=Heap[v]^=Heap[v / 2];
        Dist[v]^=Dist[v / 2]^=Dist[v]^=Dist[v / 2];
    }
    else
        break;
};

void EraseMinimum()
{
   Heap[1] = Heap[ HeapSize -- ];
   Dist[1] = Dist[ HeapSize + 1];
   for (int v = 1 ; (Dist[v + v] < Dist[v] && v + v <= HeapSize) ||
                    (Dist[v + v + 1] < Dist[v] && v + v + 1 <= HeapSize) ; )
        if ( v + v + 1 <= HeapSize && Dist[v + v + 1] < Dist[v + v] )
        {
            Heap[v]^=Heap[v + v + 1]^=Heap[v]^=Heap[v + v + 1];
            Dist[v]^=Dist[v + v + 1]^=Dist[v]^=Dist[v + v + 1];
            v += v + 1;
        } else
        {
            Heap[v]^=Heap[v + v]^=Heap[v]^=Heap[v + v];
            Dist[v]^=Dist[v + v]^=Dist[v]^=Dist[v + v];
            v += v;
        };
};

void Add(state From , state To , int Length , int CurTime)
{
  if ( To.X < 0 || To.X == N || To.Y < 0 || To.Y == M )
        return;
  if ( Length == 1 && CurTime / T[From.X][From.Y] % 2 != abs(From.Corner - To.Corner) % 2 )
    CurTime = (CurTime / T[From.X][From.Y] + 1) * T[From.X][From.Y];

  if ( D[To.X][To.Y][To.Corner] > CurTime + Length )
  {
      D[To.X][To.Y][To.Corner] = CurTime + Length;
      Insert(To , CurTime + Length);
  };
};

int main()
{
    freopen("school.in","r",stdin);
    freopen("school.out","w",stdout);
    scanf("%d %d",&M,&N);
    for (int i = 0 ; i < N ; i ++ )
        for (int j = 0 ; j < M ; j ++ )
        {
            scanf("%d",&T[i][j]);
            for (int k = 0 ; k < 4 ; k ++ )
                D[i][j][k] = Infinity;
        };

    D[N - 1][0][0] = 0;
    Insert(state(N - 1 , 0 , 0) , 0);

    for ( ; HeapSize ; )
    {
        state CurState = HashToState(Heap[1]);
        int CurTime = Dist[1];
        EraseMinimum();
        if ( CurTime > D[CurState.X][CurState.Y][CurState.Corner] )
            continue;
        if ( CurState.Corner == 0 )
        {
            Add(CurState , state(CurState.X , CurState.Y , 1) , 1 , CurTime);
            Add(CurState , state(CurState.X , CurState.Y , 2) , 1 , CurTime);
            Add(CurState , state(CurState.X + 1 , CurState.Y , 1) , 60 , CurTime);
            Add(CurState , state(CurState.X , CurState.Y - 1 , 2) , 60 , CurTime);
        };
        if ( CurState.Corner == 1 )
        {
            Add(CurState , state(CurState.X , CurState.Y , 0) , 1 , CurTime);
            Add(CurState , state(CurState.X , CurState.Y , 3) , 1 , CurTime);
            Add(CurState , state(CurState.X - 1 , CurState.Y , 0) , 60 , CurTime);
            Add(CurState , state(CurState.X , CurState.Y - 1 , 3) , 60 , CurTime);
        };
        if ( CurState.Corner == 2 )
        {
            Add(CurState , state(CurState.X , CurState.Y , 0) , 1 , CurTime);
            Add(CurState , state(CurState.X , CurState.Y , 3) , 1 , CurTime);
            Add(CurState , state(CurState.X + 1 , CurState.Y , 3) , 60 , CurTime);
            Add(CurState , state(CurState.X , CurState.Y + 1 , 0) , 60 , CurTime);
        };
        if ( CurState.Corner == 3 )
        {
            Add(CurState , state(CurState.X , CurState.Y , 1) , 1 , CurTime);
            Add(CurState , state(CurState.X , CurState.Y , 2) , 1 , CurTime);
            Add(CurState , state(CurState.X - 1 , CurState.Y , 2) , 60 , CurTime);
            Add(CurState , state(CurState.X , CurState.Y + 1 , 1) , 60 , CurTime);
        };

    };
    printf("%d\n",D[0][M - 1][3]);
    return 0;
}
